﻿using System;
using System.Diagnostics;

namespace SetAffinity {
    class Program {
        static int Main(string[] args) {
            long maxAffinity = (long)(Math.Pow(2, Environment.ProcessorCount) - 1);
            int returnVal = 0;
            if (args == null || args.Length == 0) {
                Console.WriteLine("Usage: SetAffinity <partof>processname <affinity/read>");
                Console.WriteLine("Done by Holger 'Lynxx' Hippenstiel in March.2020");
            }
            else {
                long newAffinity = 0;
                bool bAffinityGiven = false, bReadAffinity = false;
                if (args.Length > 1) {
                    if (args[1].StartsWith("%")) {
                        String strBits = args[1].Substring(1);
                        int j = strBits.Length - 1;
                        bAffinityGiven = true;
                        for (int i = 0; i < strBits.Length; i++, j--) {
                            if (strBits[i] == '1')
                                newAffinity += (long)Math.Pow(2, j);
                            else if (strBits[i] != '0') {
                                Console.WriteLine("SetAffinity-ERROR: Only 0 & 1 allowed in Bitmask !");
                                bAffinityGiven = false;
                                break;
                            }
                        }
                    }
                    else
                        bAffinityGiven = Int64.TryParse(args[1], out newAffinity);

                    if (!bAffinityGiven && args[1].ToLower().Equals("read"))
                        bReadAffinity = true;
                }

                if (!bAffinityGiven)
                    newAffinity = maxAffinity;

                if (newAffinity > maxAffinity) {
                    newAffinity &= maxAffinity;
                    Console.WriteLine("SetAffinity-ERROR: To large Affinity-Mask used, reduced to {0}/{1}", newAffinity, maxAffinity);
                }

                String strProcessName = args[0].ToLower();
                bool bAllProcesses = args[0] == "_ALL_";
                bool bProcessFound = false;

                foreach (Process oneProcess in Process.GetProcesses()) {
                    try {
                        if (oneProcess.ProcessName.ToLower().Contains(strProcessName) || bAllProcesses) {
                            bProcessFound = true;
                            long oldAffinity = (long)oneProcess.ProcessorAffinity;
                            returnVal = (int)oldAffinity;

                            if (oldAffinity == newAffinity || bReadAffinity)
                                Console.WriteLine("Process: {0} ID: {1} Affinity: {2} ({3})", oneProcess.ProcessName, oneProcess.Id, oldAffinity, getCores(oldAffinity));
                            else {
                                oneProcess.ProcessorAffinity = (IntPtr)newAffinity;
                                Console.WriteLine("Process: {0} ID: {1} Affinity: {2} ({3}) Old: {4} ({5})", oneProcess.ProcessName, oneProcess.Id, newAffinity, getCores(newAffinity), oldAffinity, getCores(oldAffinity));
                            }
                        }
                    }
                    catch (Exception) {
                        Console.WriteLine("SetAffinity-ERROR: Failed to set Affinity for Process: {0} ID: {1}", oneProcess.ProcessName, oneProcess.Id);
                    }
                }

                if (!bProcessFound)
                    Console.WriteLine("No Process found which contains '{0}'", strProcessName);
            }
            return returnVal;
        }

        static String getCores(long affinityMask) {
            String strReturn = "";
            bool bAllCores = true;
            int countCores = 0;
            for (int i = 0; i < Environment.ProcessorCount; i++) {
                if ((affinityMask & (long)Math.Pow(2, i)) != 0) {
                    strReturn += "Core" + i + ",";
                    countCores++;
                }
                else
                    bAllCores = false;
            }

            if (bAllCores)
                strReturn = "Cores: " + Environment.ProcessorCount + " All Cores";
            else {
                if (strReturn.EndsWith(","))
                    strReturn = strReturn.Substring(0, strReturn.Length - 1);

                strReturn = "Cores: " + countCores + " " + strReturn;
            }

            return strReturn;
        }
    }
}
